
import requests
import re
import os
from tqdm import tqdm
from utils.voice import get_random_voice
from utils.words import get_words


class PlayHT:
    def __init__(self, user_id, secret_key):
        self.user_id = user_id
        self.secret_key = secret_key
        self.headers = {
            'AUTHORIZATION': f'Bearer {self.secret_key}',
            'X-USER-ID': self.user_id,
            'accept': 'text/event-stream',
            'content-type': 'application/json'
        }

    def get_mp3_url(self, text, voice):
        response = requests.post("https://play.ht/api/v2/tts",
                                 headers=self.headers,
                                 json={
                                        "text": text,
                                        "voice": voice
                                 },
                                 stream=True)
        chunk_list = []
        if response.status_code == 200:
            for chunk in response.iter_content(chunk_size=3):
                if chunk:
                    chunk_list.append(chunk.decode('utf-8'))

        chunk_data = ''.join(chunk_list)
        url = re.findall(r'"url":"(.*?)"', chunk_data)
        if len(url) > 0:
            return url[0]
        else:
            return None

    def download_mp3(self, url, filename):
        doc = requests.get(url)
        with open(filename, 'wb') as f:
            f.write(doc.content)

    def get_mp3_url_and_download(self, text, voice, filename):
        url_reponse = self.get_mp3_url(text=text, voice=voice)
        if url_reponse is not None:
            self.download_mp3(url=url_reponse, filename=filename)
        else:
            error_msg = 'url_reponse is None, text: {}, voice: {}'
            raise ValueError(error_msg.format(
                    text,
                    voice,
                    filename
                )
            )

    def _tts(
        self,
        text,
        index,
        voice,
        filename,
        output_folder,
        repeat_index
    ):
        retry = 0
        if text is None or text == '':
            raise ValueError('text is None or empty')
        if voice is None:
            voice = get_random_voice()
        if filename is None:
            filename = f'{index}_{text}_{repeat_index}_{voice}.mp3'
        if output_folder is not None:
            if not os.path.exists(output_folder):
                os.makedirs(output_folder)
            filename = os.path.join(output_folder, filename)
        while retry < 3:
            try:
                self.get_mp3_url_and_download(
                    text=text,
                    voice=voice,
                    filename=filename
                )
                break
            except Exception as e:
                print(f'Error: {e}')
                retry += 1

    def tts(
        self,
        text=None,
        index=None,
        voice=None,
        filename=None,
        output_folder=None,
        repeat=2
    ):
        if repeat <= 0:
            raise ValueError('repeat must be greater than 0')
        for i in range(repeat):
            self._tts(
                text=text,
                index=index,
                voice=voice,
                filename=filename,
                output_folder=output_folder,
                repeat_index=i
                )


if __name__ == '__main__':
    ht = PlayHT(
        user_id='',
        secret_key=''
    )
    ht.tts(text='Hello World', voice='larry', filename='hello_world.mp3')

    # or
    text_list = get_words(filename='text.txt')

    for i, text in tqdm(enumerate(text_list)):
        ht.tts(text=text, index=i, output_folder='20230822', repeat=2)
